/*

*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

char* toBinary(unsigned char c);

typedef union {
   int n;
   unsigned char bytes[4];
} ProtocolInt;

typedef union {
   double x;
   unsigned char bytes[8];
} ProtocolDouble;


int main(int argc, char* argv[])
{
   FILE* logFile;
   int messages;
   int i;

   // open the "log file"
   logFile = fopen("log_file.txt","w");
   if(logFile == NULL)
   {
      fprintf(stderr, "ERROR: couldn't open %s for writing.\n", "log_file.txt");
      exit(1);
   }

   srand( (unsigned)time(NULL) ); // seed the random number generator
   messages = rand()%1000;        // random number of messages to send

   // check for a command line argument
   if (1 < argc)
   {  // send argv[1] number of messages instead
      messages = atoi( argv[1] );
   }

   // send the messages
   for (i = 0; i < messages; i++)
   {
      unsigned char messageHeader = 0;

      // "flip" a coin to to determine the "message type"
      // (either a "numeric message" or a "character message")
      if ( rand()%2 ) // send a numeric message
      {
         int i, j;
         // choose a bit pattern for ints and doubles
         messageHeader = rand() % 128;
         // send the message header
         fprintf(stdout, "%c", messageHeader);
         fflush(stdout);
         fprintf(logFile, "%s\n", toBinary(messageHeader));
         fflush(logFile);
         // send the numbers
         for (i = 0; i < 7; i++, messageHeader >>= 1)
         {
            if (0x01 & messageHeader) // send a double
            {
               ProtocolDouble d;
               // Generate a random double.
               d.x = (rand()%10000) + ((double)rand()/(double)RAND_MAX);
               fprintf(logFile, "Sending double = %.12f (0x%02x%02x%02x%02x%02x%02x%02x%02x)\n", d.x,
                                 d.bytes[7], d.bytes[6], d.bytes[5], d.bytes[4],
                                 d.bytes[3], d.bytes[2], d.bytes[1], d.bytes[0] );
               // Send the double in big-endian byte order.
               // NOTE: We are assuming here that the machine this code is
               // running on is little-endian (i.e., an Intel processor).
               for (j = 7; j >= 0; j--)
               {
                  fprintf(stdout,  "%c",     d.bytes[j]);
                  fprintf(logFile, "%02x  ", d.bytes[j]);
               }
               fprintf(logFile, "\n");
               fflush(stdout);
               fflush(logFile);
            }
            else  // send an int
            {
               ProtocolInt k;
               // Generate a random int.
               k.n = rand();
               fprintf(logFile, "Sending int = %d (%#010x)\n", k.n, k.n);
               // Send the int in weird-endian byte order.
               // NOTE: We are assuming here that the machine this code is
               // running on is little-endian (i.e., an Intel processor).
               fprintf(stdout,  "%c",     k.bytes[2]);
               fprintf(logFile, "%02x ",  k.bytes[2]);
               fprintf(stdout,  "%c",     k.bytes[1]);
               fprintf(logFile, "%02x ",  k.bytes[1]);
               fprintf(stdout,  "%c",     k.bytes[3]);
               fprintf(logFile, "%02x ",  k.bytes[3]);
               fprintf(stdout,  "%c",     k.bytes[0]);
               fprintf(logFile, "%02x",   k.bytes[0]);
               fprintf(logFile, "\n");
               fflush(stdout);
               fflush(logFile);
            }
         }
      }
      else // send a character message
      {
         int i;
         // number of characters in this message
         unsigned char count = rand() % 128;
         // make the "message header"
         messageHeader = 0x80 | count;
         // send the message header
         fprintf(stdout, "%c", messageHeader);
         fflush(stdout);
         fprintf(logFile, "%s (%d characters)\n", toBinary(messageHeader), 0x7f & count);
         fflush(logFile);
         // send the characters
         for (i = 0; i < count; i++)
         {
            // a random printable character (see an ascii table)
            char rndChar = 32 + (rand() % 95);
            fprintf(stdout, "%c", rndChar);
            fflush(stdout);
            fprintf(logFile, "%c", rndChar);
            fflush(logFile);
         }
         fprintf(logFile, "\n");
         fflush(logFile);
      }// send a character message
      fprintf(logFile, "\n");
      fflush(logFile);
   }// send the messages

   // send a "end of data" message
   fprintf(stdout, "%c", 0x80);
   fflush(stdout);
   fprintf(logFile, "%s", toBinary(0x80));
   fflush(logFile);

   fclose(logFile);

   return 0;
}//main()


char* toBinary(unsigned char c)
{
   static char digits[9];
   int i;
   for (i = 0; i < 8; i++)
   {
      if ( c & 0x80 )
      {
         digits[i] = '1';
      }
      else
      {
         digits[i] = '0';
      }
      c <<= 1;
   }
   digits[8] = '\0';  // null terminator
   return digits;
}//toBinary()
